#!/bin/bash
#
# script for starting SATOSA with Gunicorn with a set of SSL/TLS files

set -eo pipefail

SATOSA_GUNICORN_KEY="${SATOSA_GUNICORN_KEY:-ssl/gunicorn.key}"
SATOSA_GUNICORN_CERT="${SATOSA_GUNICORN_CERT:-ssl/gunicorn.crt}"
SATOSA_CONFIG="${SATOSA_CONFIG:-$(pwd)/proxy_conf.yaml}"
SATOSA_USE_PIPENV=${SATOSA_USE_PIPENV:-1}
SATOSA_PORT="${SATOSA_PORT:-8443}"
export SATOSA_CONFIG

# check which encryption files to use
if [[ ! -f "${SATOSA_GUNICORN_KEY}" ]]; then
    echo >&2 "WARN: using test key!"
    SATOSA_GUNICORN_KEY="ssl/test.key"
fi

if [[ ! -f "${SATOSA_GUNICORN_CERT}" ]]; then
    echo >&2 "WARN: using test certificate!"
    SATOSA_GUNICORN_CERT="ssl/test.crt"
fi

# make sure the signing certificate for the SAML metadata is available
if [[ -z "${SATOSA_SAML_METADATA_CERT_FILE}" ]]; then
    echo >&2 "WARN: no metadata signing key specified, signature validation will be skipped!"

elif [[ "${SATOSA_SAML_METADATA_CERT_FILE}" =~ ^https:// ]]; then
    if [[ -z "${SATOSA_SAML_METADATA_CERT_FINGERPRINT}" ]]; then
        echo >&2 "WARN: blindly fetching metadata certificate from '${SATOSA_SAML_METADATA_CERT_FILE}'!"
    fi

    curl -fso "config/saml-metadata-signing.crt" "${SATOSA_SAML_METADATA_CERT_FILE}"
    SATOSA_SAML_METADATA_CERT_FILE="config/saml-metadata-signing.crt"
    export SATOSA_SAML_METADATA_CERT_FILE
fi

# check the certificate's fingerprint
if [[ -n "${SATOSA_SAML_METADATA_CERT_FINGERPRINT}" ]]; then
    fingerprint=$(openssl x509 -noout -fingerprint -sha256 -in "${SATOSA_SAML_METADATA_CERT_FILE}" | cut -d= -f2)
    if [[ "${fingerprint}" != "${SATOSA_SAML_METADATA_CERT_FINGERPRINT}" ]]; then
        echo >&2 "ERROR: calculated certificate fingerprint deviates from the expected value!"
        echo >&2 "expected:   ${SATOSA_SAML_METADATA_CERT_FINGERPRINT}"
        echo >&2 "calculated: ${fingerprint}"
        exit 1
    fi
fi

if [[ ! -f "oidc-clients.json" ]]; then
    echo >&2 "ERROR: could not find the file 'oidc-clients.json'!"
    exit 1
fi

# either use gunicorn directly, or run it through pipenv
if [[ "${SATOSA_USE_PIPENV}" -ne "1" ]]; then
    gunicorn -b "0.0.0.0:${SATOSA_PORT}" satosa.wsgi:app --keyfile="${SATOSA_GUNICORN_KEY}" --certfile="${SATOSA_GUNICORN_CERT}"
else
    pipenv run gunicorn -b "0.0.0.0:${SATOSA_PORT}" satosa.wsgi:app --keyfile="${SATOSA_GUNICORN_KEY}" --certfile="${SATOSA_GUNICORN_CERT}"
fi
